'use strict';

var template = require('@babel/template');
var t = require('@babel/types');
var helpers = require('@native-twin/helpers');
var babel_predicates = require('./babel.predicates');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var template__default = /*#__PURE__*/_interopDefaultCompat(template);
var t__namespace = /*#__PURE__*/_interopNamespaceCompat(t);

const createPrimitiveExpression = (value) => {
    if (typeof value === 'string')
        return t__namespace.stringLiteral(value);
    if (typeof value === 'number')
        return t__namespace.numericLiteral(value);
    return t__namespace.booleanLiteral(value);
};
const createObjectExpressionWithTemplate = (expression) => {
    const ast = template__default.default.expression(JSON.stringify(expression));
    return ast();
};
const createObjectExpressionAST = (obj) => {
    if (Array.isArray(obj)) {
        return t__namespace.arrayExpression(createArrayExpression(obj));
    }
    return t__namespace.objectExpression(createOjectExpressionProperties(Object.entries(obj)));
};
const createArrayExpression = (x, results = []) => {
    const [next, ...rest] = x;
    if (!next)
        return results;
    let value = null;
    if (babel_predicates.valueIsPrimitive(next)) {
        value = createPrimitiveExpression(next);
    }
    if (helpers.isObject(next) || Array.isArray(next)) {
        value = createObjectExpressionAST(next);
    }
    if (value) {
        results.push(value);
    }
    return createArrayExpression(rest, results);
};
const createOjectExpressionProperties = (entries, results = []) => {
    const [next, ...rest] = entries;
    if (!next) {
        return results;
    }
    const key = t__namespace.stringLiteral(next[0]);
    let value = null;
    if (babel_predicates.valueIsPrimitive(next[1])) {
        value = createPrimitiveExpression(next[1]);
    }
    if (helpers.isObject(next[1]) || Array.isArray(next[1])) {
        value = createObjectExpressionAST(next[1]);
    }
    if (value) {
        results.push(t__namespace.objectProperty(key, value));
    }
    return createOjectExpressionProperties(rest, results);
};
const templateLiteralToStringLike = (literal) => {
    const strings = literal.quasis
        .map((x) => (x.value.cooked ? x.value.cooked : x.value.raw))
        .map((x) => x.trim().replace(/\n/g, '').trim().replace(/\s+/g, ' '))
        .filter((x) => x.length > 0)
        .join('');
    const expressions = t__namespace.templateLiteral(literal.quasis.map(() => t__namespace.templateElement({ raw: '', cooked: '' })), literal.expressions);
    return { strings, expressions: expressions };
};

exports.createArrayExpression = createArrayExpression;
exports.createObjectExpressionAST = createObjectExpressionAST;
exports.createObjectExpressionWithTemplate = createObjectExpressionWithTemplate;
exports.createOjectExpressionProperties = createOjectExpressionProperties;
exports.createPrimitiveExpression = createPrimitiveExpression;
exports.templateLiteralToStringLike = templateLiteralToStringLike;
//# sourceMappingURL=babel.constructors.js.map
