import template from '@babel/template';
import * as t from '@babel/types';
import { isObject } from '@native-twin/helpers';
import { valueIsPrimitive } from './babel.predicates';

const createPrimitiveExpression = (value) => {
    if (typeof value === 'string')
        return t.stringLiteral(value);
    if (typeof value === 'number')
        return t.numericLiteral(value);
    return t.booleanLiteral(value);
};
const createObjectExpressionWithTemplate = (expression) => {
    const ast = template.expression(JSON.stringify(expression));
    return ast();
};
const createObjectExpressionAST = (obj) => {
    if (Array.isArray(obj)) {
        return t.arrayExpression(createArrayExpression(obj));
    }
    return t.objectExpression(createOjectExpressionProperties(Object.entries(obj)));
};
const createArrayExpression = (x, results = []) => {
    const [next, ...rest] = x;
    if (!next)
        return results;
    let value = null;
    if (valueIsPrimitive(next)) {
        value = createPrimitiveExpression(next);
    }
    if (isObject(next) || Array.isArray(next)) {
        value = createObjectExpressionAST(next);
    }
    if (value) {
        results.push(value);
    }
    return createArrayExpression(rest, results);
};
const createOjectExpressionProperties = (entries, results = []) => {
    const [next, ...rest] = entries;
    if (!next) {
        return results;
    }
    const key = t.stringLiteral(next[0]);
    let value = null;
    if (valueIsPrimitive(next[1])) {
        value = createPrimitiveExpression(next[1]);
    }
    if (isObject(next[1]) || Array.isArray(next[1])) {
        value = createObjectExpressionAST(next[1]);
    }
    if (value) {
        results.push(t.objectProperty(key, value));
    }
    return createOjectExpressionProperties(rest, results);
};
const templateLiteralToStringLike = (literal) => {
    const strings = literal.quasis
        .map((x) => (x.value.cooked ? x.value.cooked : x.value.raw))
        .map((x) => x.trim().replace(/\n/g, '').trim().replace(/\s+/g, ' '))
        .filter((x) => x.length > 0)
        .join('');
    const expressions = t.templateLiteral(literal.quasis.map(() => t.templateElement({ raw: '', cooked: '' })), literal.expressions);
    return { strings, expressions: expressions };
};

export { createArrayExpression, createObjectExpressionAST, createObjectExpressionWithTemplate, createOjectExpressionProperties, createPrimitiveExpression, templateLiteralToStringLike };
//# sourceMappingURL=babel.constructors.js.map
