import * as t from '@babel/types';
import * as Option from 'effect/Option';

const maybeCallExpression = (node) => {
    return node.pipe(Option.flatMap((x) => t.isCallExpression(x.init) ? Option.some([x, x.init]) : Option.none()));
};
const maybeImportDeclaration = (binding) => {
    return binding.pipe(Option.flatMap((x) => x.path.parentPath &&
        t.isImportDeclaration(x.path.parentPath.node) &&
        x.path.parentPath.node.source.value.toLowerCase() === 'react'
        ? Option.some(true)
        : Option.none()));
};
const maybeReactIdent = (node) => {
    return node.pipe(Option.flatMap((x) => {
        if (t.isIdentifier(x.object, { name: 'react' }) ||
            t.isIdentifier(x.object, { name: 'React' })) {
            return Option.some(x.object);
        }
        if (t.isMemberExpression(x.object) &&
            t.isIdentifier(x.object.object, { name: '_react' }) &&
            t.isIdentifier(x.object.property, { name: 'default' })) {
            return Option.some(x.object.object);
        }
        return Option.none();
    }));
};
const maybeCreateElementExpression = (path) => {
    if (t.isIdentifier(path.node.property, { name: 'createElement' })) {
        return Option.some(path.node);
    }
    return Option.none();
};
const maybeBinding = (node, path) => {
    return node.pipe(Option.flatMap((a) => Option.fromNullable(path.scope.getBinding(a.name))));
};
const maybeVariableDeclarator = (binding) => {
    return binding.pipe(Option.flatMap((x) => x.path.isVariableDeclarator() ? Option.some(x.path.node) : Option.none()));
};

export { maybeBinding, maybeCallExpression, maybeCreateElementExpression, maybeImportDeclaration, maybeReactIdent, maybeVariableDeclarator };
//# sourceMappingURL=babel.maps.js.map
