import * as t from '@babel/types';
import * as Console from 'effect/Console';
import * as Option from 'effect/Option';
import { maybeVariableDeclarator, maybeCallExpression, isReactRequire, isReactInteropRequire } from '../babel';

const debugEffect = (x) => Console.log(x);
const isCreateElementIdent = (node) => {
    return t.isIdentifier(node.property, { name: 'createElement' });
};
const getReactIdent = (x) => {
    if (t.isIdentifier(x.node.object, { name: 'react' }) ||
        t.isIdentifier(x.node.object, { name: 'React' })) {
        return x.node.object.name;
    }
    if (t.isMemberExpression(x.node.object) &&
        t.isIdentifier(x.node.object.object, { name: '_react' }) &&
        t.isIdentifier(x.node.object.property, { name: 'default' })) {
        return x.node.object.object.name;
    }
    return null;
};
const isReactRequireBinding = (x) => {
    if (x.path.isVariableDeclarator() && t.isCallExpression(x.path.node.init)) {
        if (t.isIdentifier(x.path.node.init.callee, { name: 'require' }) &&
            t.isStringLiteral(x.path.node.init.arguments[0], { value: 'react' })) {
            return true;
        }
        else if (
        // const <name> = _interopRequireDefault(require("react"))
        t.isIdentifier(x.path.node.init.callee, { name: '_interopRequireDefault' }) &&
            t.isCallExpression(x.path.node.init.arguments[0]) &&
            t.isIdentifier(x.path.node.init.arguments[0].callee, { name: 'require' }) &&
            t.isStringLiteral(x.path.node.init.arguments[0].arguments[0], {
                value: 'react',
            })) {
            return true;
        }
    }
    return false;
};
const isReactImport = (x) => {
    if (x.path.isImportSpecifier() ||
        x.path.isImportDefaultSpecifier() ||
        x.path.isImportDeclaration() ||
        x.path.isImportNamespaceSpecifier()) {
        return (t.isImportDeclaration(x.path.parentPath.node) &&
            x.path.parentPath.node.source.value.toLowerCase() === 'react');
    }
    return false;
};
const pipeMemberExpression = (self) => {
    return Option.Do.pipe(() => (isCreateElementIdent(self.node) ? Option.some(self) : Option.none()), Option.flatMapNullable(getReactIdent), Option.flatMapNullable((x) => self.scope.getBinding(x)), Option.map((x) => isReactRequireBinding(x) || isReactImport(x)), Option.getOrElse(() => false));
};
const maybeBindingIsReactImport = (x) => x.pipe(maybeVariableDeclarator, maybeCallExpression, (node) => [isReactRequire(node), isReactInteropRequire(node)], Option.firstSomeOf);

export { debugEffect, getReactIdent, isReactImport, isReactRequireBinding, maybeBindingIsReactImport, pipeMemberExpression };
//# sourceMappingURL=import.utils.js.map
