import * as Context from 'effect/Context';
import { pipe } from 'effect/Function';
import * as Layer from 'effect/Layer';
import * as MutableHashMap from 'effect/MutableHashMap';
import * as Option from 'effect/Option';
import { TwinBabelFile } from '../models/BabelFile.model';

const babelCache = MutableHashMap.empty();
const getOrSetBabelFile = (file, tree) => {
    return pipe(babelCache, MutableHashMap.get(file.filename), Option.getOrElse(() => {
        const newFile = new TwinBabelFile(file, tree);
        pipe(babelCache, MutableHashMap.set(file.filename, newFile));
        return newFile;
    }));
};
class BabelCacheContext extends Context.Tag('transformer/babel-cache-context')() {
    static Live = Layer.succeed(BabelCacheContext, {
        cache: babelCache,
        getOrSetBabelFile,
    });
}

export { BabelCacheContext, babelCache };
//# sourceMappingURL=babel.cache.js.map
