'use strict';

var t = require('@babel/types');
var jsx = require('@native-twin/css/jsx');
var babel = require('../../babel');
var twin = require('../twin');

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var t__namespace = /*#__PURE__*/_interopNamespaceCompat(t);

const createRequireExpression = (path) => {
    return t__namespace.callExpression(t__namespace.identifier('require'), [t__namespace.stringLiteral(path)]);
};
const createJsxAttribute = (name, value) => {
    const expression = babel.createPrimitiveExpression(value);
    return t__namespace.jsxAttribute(t__namespace.jsxIdentifier(name), t__namespace.jsxExpressionContainer(expression));
};
const addJsxAttribute = (element, name, value) => {
    if (!t__namespace.isJSXElement(element))
        return;
    const newAttribute = createJsxAttribute(name, value);
    if (babel.hasJsxAttribute(element, name, value)) {
        element.openingElement.attributes = element.openingElement.attributes.map((x) => {
            if (x.type === 'JSXSpreadAttribute')
                return x;
            if (x.type === 'JSXAttribute' &&
                x.name.type === 'JSXIdentifier' &&
                x.name.name === name) {
                return newAttribute;
            }
            return x;
        });
        return;
    }
    element.openingElement.attributes.push(newAttribute);
};
const addJsxExpressionAttribute = (element, name, value) => {
    if (!t__namespace.isJSXElement(element))
        return;
    const newAttribute = t__namespace.jsxAttribute(t__namespace.jsxIdentifier(name), t__namespace.jsxExpressionContainer(value));
    if (JSXElementHasAttribute(element, name)) {
        console.log('ALREADY_HAS: ', name);
        element.openingElement.attributes = element.openingElement.attributes.map((x) => {
            if (x.type === 'JSXSpreadAttribute')
                return x;
            if (x.type === 'JSXAttribute' &&
                x.name.type === 'JSXIdentifier' &&
                x.name.name === name) {
                return newAttribute;
            }
            return x;
        });
        return;
    }
    element.openingElement.attributes.push(newAttribute);
};
const JSXElementHasAttribute = (element, name) => {
    return element.openingElement.attributes.some((x) => x.type === 'JSXAttribute' &&
        x.name.type === 'JSXIdentifier' &&
        x.name.name === name);
};
function addTwinPropsToElement(elementNode, entries, options) {
    const stringEntries = twin.entriesToComponentData(elementNode.id, jsx.getRawSheet(entries));
    const astProps = twin.runtimeEntriesToAst(stringEntries.styledProp);
    // const treeProp = elementNodeToTree(elementNode, filename, elementNode.);
    if (options.componentID) {
        addJsxAttribute(elementNode.path, '_twinComponentID', elementNode.id);
    }
    if (options.order) {
        addJsxAttribute(elementNode.path, '_twinOrd', elementNode.order);
    }
    if (options.styledProps && astProps) {
        addJsxExpressionAttribute(elementNode.path, '_twinComponentSheet', astProps);
    }
    if (options.templateStyles && stringEntries.templateEntries) {
        const astTemplate = twin.runtimeEntriesToAst(stringEntries.templateEntries);
        if (astTemplate) {
            addJsxExpressionAttribute(elementNode.path, '_twinComponentTemplateEntries', astTemplate);
        }
    }
    return astProps;
}
function compileCssForElement(elementNode, entries, options) {
    // const stringEntries = entriesToObject(elementNode.id, getRawSheet(entries));
    // const astProps = runtimeEntriesToAst(stringEntries.styledProp);
    // const treeProp = elementNodeToTree(elementNode, filename, elementNode.);
    // const astTemplate = runtimeEntriesToAst(stringEntries.templateEntries);
    //
    // if (options.componentID) {
    //   addJsxAttribute(elementNode.path, '_twinComponentID', elementNode.id);
    // }
    // if (options.order) {
    //   addJsxAttribute(elementNode.path, '_twinOrd', elementNode.order);
    // }
}

exports.JSXElementHasAttribute = JSXElementHasAttribute;
exports.addJsxAttribute = addJsxAttribute;
exports.addJsxExpressionAttribute = addJsxExpressionAttribute;
exports.addTwinPropsToElement = addTwinPropsToElement;
exports.compileCssForElement = compileCssForElement;
exports.createJsxAttribute = createJsxAttribute;
exports.createRequireExpression = createRequireExpression;
//# sourceMappingURL=jsx.builder.js.map
