'use strict';

var traverse = require('@babel/traverse');
var t = require('@babel/types');
var RA = require('effect/Array');
var Effect = require('effect/Effect');
var Function = require('effect/Function');
var Hash = require('effect/Hash');
var Option = require('effect/Option');
var jsx = require('@native-twin/css/jsx');
var tree = require('@native-twin/helpers/tree');
var component_maps = require('../../utils/component.maps');
var jsxPredicates = require('./jsx.predicates');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var traverse__default = /*#__PURE__*/_interopDefaultCompat(traverse);
var t__namespace = /*#__PURE__*/_interopNamespaceCompat(t);
var RA__namespace = /*#__PURE__*/_interopNamespaceCompat(RA);
var Effect__namespace = /*#__PURE__*/_interopNamespaceCompat(Effect);
var Hash__namespace = /*#__PURE__*/_interopNamespaceCompat(Hash);
var Option__namespace = /*#__PURE__*/_interopNamespaceCompat(Option);
var jsxPredicates__namespace = /*#__PURE__*/_interopNamespaceCompat(jsxPredicates);

const getJSXElementSource = (path) => Function.pipe(getJSXElementName(path.node.openingElement), Option__namespace.flatMap((x) => Option__namespace.fromNullable(path.scope.getBinding(x))), Option__namespace.flatMap((binding) => getBindingImportSource(binding)), Option__namespace.getOrElse(() => ({ kind: 'local', source: 'unknown' })));
const getBindingImportDeclaration = (binding) => Function.pipe(binding.path, Option__namespace.liftPredicate(jsxPredicates__namespace.isImportSpecifier), Option__namespace.bindTo('importSpecifier'), Option__namespace.bind('importDeclaration', ({ importSpecifier }) => Option__namespace.liftPredicate(importSpecifier.parentPath, jsxPredicates__namespace.isImportDeclaration)), Option__namespace.map((source) => ({
    kind: 'import',
    source: source.importDeclaration.node.source.value,
})));
const getBindingRequireDeclaration = (binding) => Function.pipe(binding.path, Option__namespace.liftPredicate(jsxPredicates__namespace.isVariableDeclaratorPath), Option__namespace.bindTo('importSpecifier'), Option__namespace.bind('requireExpression', ({ importSpecifier }) => Function.pipe(Option__namespace.fromNullable(importSpecifier.node.init), Option__namespace.flatMap((init) => Option__namespace.liftPredicate(init, jsxPredicates__namespace.isCallExpression)), Option__namespace.flatMap((x) => RA__namespace.head(x.arguments)), Option__namespace.flatMap((x) => Option__namespace.liftPredicate(x, t__namespace.isStringLiteral)))), Option__namespace.map((source) => {
    return {
        kind: 'require',
        source: source.requireExpression.value,
    };
}));
const getBindingImportSource = (binding) => Function.pipe([getBindingImportDeclaration(binding), getBindingRequireDeclaration(binding)], Option__namespace.firstSomeOf);
/**
 * @internal
 * @category Transformer
 * @domain Babel
 * Extract {@link JSXMappedAttribute[]} list from a jsx Attribute
 * */
const getJSXMappedAttributes = (attributes, config) => {
    return attributes
        .map((x) => extractStyledProp(x, config))
        .filter((x) => x !== null);
};
/**
 * @category Transformer
 * @domain Babel
 * Extract {@link JSXMappedAttribute[]} list from a jsx Attribute
 * */
const extractMappedAttributes = (node) => {
    const attributes = getJSXElementAttrs(node);
    return Function.pipe(getJSXElementName(node.openingElement), Option__namespace.flatMap((x) => Option__namespace.fromNullable(getJSXElementConfig(x))), Option__namespace.map((mapped) => getJSXMappedAttributes(attributes, mapped)), Option__namespace.getOrElse(() => []));
};
/**
 * * @internal
 * @domain Babel
 * @description Extract the {@link MappedComponent} from any {@link ValidJSXElementNode}
 * */
const getJSXElementConfig = (tagName) => {
    const componentConfig = component_maps.mappedComponents.find((x) => x.name === tagName);
    if (!componentConfig)
        return component_maps.createCommonMappedAttribute(tagName);
    return componentConfig;
};
/**
 * @domain Babel
 * @description Extract the {@link t.JSXAttribute[]} from any {@link t.JSXElement}
 * */
const getJSXElementAttrs = (element) => Function.pipe(element.openingElement.attributes, RA__namespace.filter(jsxPredicates__namespace.isJSXAttribute));
/**
 * @domain Babel
 * @description Extract the {@link JSXMappedAttribute} from any {@link t.JSXAttribute}
 * */
const extractStyledProp = (attribute, config) => {
    const validClassNames = Object.entries(config.config);
    if (!t__namespace.isJSXAttribute(attribute))
        return null;
    if (!t__namespace.isJSXIdentifier(attribute.name))
        return null;
    const className = validClassNames.find((x) => attribute.name.name === x[0]);
    if (!className)
        return null;
    if (t__namespace.isStringLiteral(attribute.value)) {
        return {
            prop: className[0],
            target: className[1],
            value: attribute.value,
        };
    }
    if (t__namespace.isJSXExpressionContainer(attribute.value)) {
        if (t__namespace.isTemplateLiteral(attribute.value.expression)) {
            return {
                prop: className[0],
                target: className[1],
                value: attribute.value.expression,
            };
        }
        if (t__namespace.isCallExpression(attribute.value.expression)) {
            return {
                prop: className[0],
                target: className[1],
                value: t__namespace.templateLiteral([
                    t__namespace.templateElement({ raw: '', cooked: '' }),
                    t__namespace.templateElement({ raw: '', cooked: '' }),
                ], [attribute.value.expression]),
            };
        }
    }
    return null;
};
const getJSXElementName = (openingElement) => {
    if (t__namespace.isJSXIdentifier(openingElement.name)) {
        return Option__namespace.some(openingElement.name.name);
    }
    return Option__namespace.none();
};
const getBabelJSXElementChildsCount = (node) => Function.pipe(node.children, RA__namespace.filterMap((x) => Function.pipe(x, Option__namespace.liftPredicate(t__namespace.isJSXElement))), RA__namespace.length);
const getAstTrees = (ast, filename) => {
    return Effect__namespace.async((resolve) => {
        const cssImports = [];
        traverse__default.default(ast, {
            Program: {
                exit() {
                    resolve(Effect__namespace.succeed(this.trees));
                },
            },
            ImportDeclaration(path) {
                if (path.node.source.value.endsWith('.css')) {
                    cssImports.push(path.node.source.value);
                }
            },
            JSXElement(path) {
                const uid = path.scope.generateUidIdentifier();
                // console.log('UID: ', uid.name, uidUnNamed.name);
                const hash = Hash__namespace.string(filename + uid.name);
                const parentTree = new tree.Tree({
                    order: -1,
                    babelNode: path.node,
                    uid: `${hash}__${uid.name}:__1`,
                    source: getJSXElementSource(path),
                    cssImports: cssImports,
                    parentID: null,
                });
                gelBabelJSXElementChildLeaves(path, parentTree.root);
                this.trees.push(parentTree);
                path.skip();
            },
        }, undefined, {
            trees: [],
        });
    });
};
const gelBabelJSXElementChildLeaves = (path, parent) => {
    const childs = Function.pipe(path.get('children'), RA__namespace.filterMap(Option__namespace.liftPredicate(jsxPredicates__namespace.isJSXElementPath)));
    for (const childPath of childs) {
        const order = parent.childrenCount;
        const childUid = path.scope.generateUid();
        // console.log('CHILD_UID: ', childUid);
        const childLeave = parent.addChild({
            order,
            babelNode: childPath.node,
            uid: `${parent.value.uid}:${order}____${childUid}`,
            source: getJSXElementSource(childPath),
            cssImports: parent.value.cssImports,
            parentID: parent.value.uid,
        });
        childLeave.parent = parent;
        gelBabelJSXElementChildLeaves(childPath, childLeave);
    }
};
const getJSXCompiledTreeRuntime = (leave, parentLeave) => {
    const runtimeSheet = Function.pipe(parentLeave, Option__namespace.map((parent) => jsx.applyParentEntries(leave.entries, parent.childEntries, leave.order, leave.parentSize)), Option__namespace.getOrElse(() => leave.entries));
    return {
        leave,
        runtimeSheet,
    };
};

exports.extractMappedAttributes = extractMappedAttributes;
exports.extractStyledProp = extractStyledProp;
exports.gelBabelJSXElementChildLeaves = gelBabelJSXElementChildLeaves;
exports.getAstTrees = getAstTrees;
exports.getBabelJSXElementChildsCount = getBabelJSXElementChildsCount;
exports.getBindingImportSource = getBindingImportSource;
exports.getJSXCompiledTreeRuntime = getJSXCompiledTreeRuntime;
exports.getJSXElementAttrs = getJSXElementAttrs;
exports.getJSXElementName = getJSXElementName;
exports.getJSXElementSource = getJSXElementSource;
//# sourceMappingURL=jsx.maps.js.map
