'use strict';

var generate = require('@babel/generator');
var RA = require('effect/Array');
var Effect = require('effect/Effect');
var Function = require('effect/Function');
var HashMap = require('effect/HashMap');
var Option = require('effect/Option');
var Stream = require('effect/Stream');
var babel = require('../../babel');
var models = require('../../models');
var services = require('../../services');
var NativeTwin_service = require('../../services/NativeTwin.service');
var jsx_builder = require('./jsx.builder');
var jsx_debug = require('./jsx.debug');
var jsx_maps = require('./jsx.maps');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var generate__default = /*#__PURE__*/_interopDefaultCompat(generate);
var RA__namespace = /*#__PURE__*/_interopNamespaceCompat(RA);
var Effect__namespace = /*#__PURE__*/_interopNamespaceCompat(Effect);
var HashMap__namespace = /*#__PURE__*/_interopNamespaceCompat(HashMap);
var Option__namespace = /*#__PURE__*/_interopNamespaceCompat(Option);
var Stream__namespace = /*#__PURE__*/_interopNamespaceCompat(Stream);

const transformJSXFile = (code) => {
    return Effect__namespace.gen(function* () {
        const transformer = yield* services.BabelTransformerService;
        const ctx = yield* services.MetroCompilerContext;
        const ast = babel.createBabelAST(code);
        const babelTrees = yield* jsx_maps.getAstTrees(ast, ctx.filename);
        const registry = yield* Function.pipe(Stream__namespace.fromIterable(babelTrees), Stream__namespace.mapEffect((x) => extractSheetsFromTree(x, ctx.filename)), Stream__namespace.map(HashMap__namespace.fromIterable), Stream__namespace.runFold(HashMap__namespace.empty(), (prev, current) => {
            return Function.pipe(prev, HashMap__namespace.union(current));
        }));
        const trees = yield* Effect__namespace.if(Effect__namespace.sync(() => ctx.platform !== 'web'), {
            onFalse: () => Effect__namespace.sync(() => {
                return HashMap__namespace.empty();
            }),
            onTrue: () => Effect__namespace.sync(() => {
                return transformer.transformLeave(registry);
            }),
        });
        const devToolsTree = Function.pipe(jsx_debug.createDevToolsTree(trees, babelTrees), RA__namespace.fromIterable, RA__namespace.map((x) => {
            if (ctx.platform !== 'web') {
                const elementTree = jsx_debug.elementNodeToTree(x, x.leave.filename);
                jsx_builder.addJsxExpressionAttribute(x.leave.path, '_twinComponentTree', elementTree);
            }
            return x;
        }));
        let generatedCode = Function.pipe(Option__namespace.fromNullable(generate__default.default(ast)), Option__namespace.map((x) => x.code), Option__namespace.getOrElse(() => code));
        const classNames = Function.pipe(HashMap__namespace.values(transformer.transformLeave(registry)), RA__namespace.fromIterable, RA__namespace.flatMap((x) => x.runtimeSheet), RA__namespace.map((x) => x.classNames), RA__namespace.join('\n'));
        return {
            trees,
            devToolsTree,
            classNames,
            generated: generatedCode,
            cssImports: Function.pipe(trees, HashMap__namespace.values, RA__namespace.fromIterable, RA__namespace.flatMap((x) => x.leave.leave.value.cssImports), RA__namespace.dedupe),
        };
    });
};
const extractSheetsFromTree = (tree, fileName) => Effect__namespace.gen(function* () {
    // const fileSheet = MutableHashMap.empty<string, CompiledTree>();
    const twin = yield* NativeTwin_service.NativeTwinService;
    console.groupEnd();
    const fileSheet = RA__namespace.empty();
    tree.traverse((leave) => {
        const { value } = leave;
        const runtimeData = jsx_maps.extractMappedAttributes(leave.value.babelNode);
        const model = new models.JSXElementNode({
            leave,
            order: value.order,
            filename: fileName,
            runtimeData,
            twin,
        });
        fileSheet.push([model.id, model]);
    }, 'breadthFirst');
    return fileSheet;
});

exports.transformJSXFile = transformJSXFile;
//# sourceMappingURL=jsx.visitors.js.map
