'use strict';

var Option = require('effect/Option');
var fs = require('node:fs');
var path = require('node:path');
var NativeTwin = require('@native-twin/core');
var constants = require('../../constants');
var loadJs = require('../../utils/load-js');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var Option__namespace = /*#__PURE__*/_interopNamespaceCompat(Option);
var fs__default = /*#__PURE__*/_interopDefaultCompat(fs);
var path__default = /*#__PURE__*/_interopDefaultCompat(path);
var NativeTwin__namespace = /*#__PURE__*/_interopNamespaceCompat(NativeTwin);

let tw = null;
function loadNativeTwin() {
    return loadJs.requireJS('@native-twin/core');
}
function loadVirtualSheet() {
    return loadJs.requireJS('@native-twin/css').createVirtualSheet();
}
function loadNativeTwinConfig(mod, config, options) {
    return mod.createTailwind(config, loadVirtualSheet());
}
function getUserTwinConfig(rootDir, options) {
    const twinConfigPath = getTwinConfigPath(rootDir, options);
    return twinConfigPath.pipe(Option__namespace.map((x) => loadJs.requireJS(x)), Option__namespace.map((x) => NativeTwin__namespace.defineConfig({
        ...x,
        mode: options.platform === 'web' ? 'web' : x.mode ?? 'native',
        root: { rem: x.root.rem ?? 16 },
    })), Option__namespace.getOrElse(() => NativeTwin__namespace.defineConfig({
        content: [],
        mode: options.platform === 'web' ? 'web' : 'native',
        root: { rem: 16 },
    })));
}
const getTwinConfigPath = (rootDir, options) => {
    let twPath;
    if (options.twinConfigPath) {
        twPath = path__default.default.join(rootDir, options.twinConfigPath);
    }
    if (!twPath) {
        twPath = constants.TWIN_DEFAULT_FILES.map((x) => path__default.default.join(rootDir, x)).find((x) => fs__default.default.existsSync(x));
    }
    return Option__namespace.fromNullable(twPath).pipe(Option__namespace.map((x) => path__default.default.resolve(x)));
};
function setupNativeTwin(config, options) {
    if (tw) {
        return tw;
    }
    const nativeTwin = loadNativeTwin();
    tw = loadNativeTwinConfig(nativeTwin, config);
    return tw;
}
const createTwinCSSFiles = ({ outputDir, inputCSS, }) => {
    if (!fs__default.default.existsSync(path__default.default.resolve(outputDir))) {
        fs__default.default.mkdirSync(outputDir, { recursive: true });
    }
    if (!inputCSS) {
        inputCSS = path__default.default.join(outputDir, constants.TWIN_INPUT_CSS_FILE);
        fs__default.default.writeFileSync(inputCSS, '');
    }
    // fs.writeFileSync(outputCSS, '.tt_root { font-size: 16px }', 'utf-8');
    return {
        inputCSS,
    };
};
// export const createCacheDir = (outputDir: string) => {
//   if (!fs.existsSync(path.resolve(outputDir))) {
//     fs.mkdirSync(outputDir, { recursive: true });
//   }
//   fs.writeFileSync(path.join(outputDir, TWIN_STYLES_FILE), twinModuleExportString);
// };
const deleteCacheDir = (outputDir) => {
    if (!fs__default.default.existsSync(path__default.default.resolve(outputDir))) {
        fs__default.default.rmdirSync(outputDir, { recursive: true });
    }
};
const getTwinCacheDir = () => path__default.default.join(path__default.default.dirname(require.resolve('@native-twin/core')), '.cache');

exports.createTwinCSSFiles = createTwinCSSFiles;
exports.deleteCacheDir = deleteCacheDir;
exports.getTwinCacheDir = getTwinCacheDir;
exports.getUserTwinConfig = getUserTwinConfig;
exports.setupNativeTwin = setupNativeTwin;
//# sourceMappingURL=twin.setup.js.map
